/************************************************************************
 *
 *  Module  ttime_api.h
 *
 *  Author  SIS
 *
 *  Description
 *
 *      This header file contains data necessary for the API to the
 *      True Time board. this contains all of the structure definitions
 *      for the individual registers.
 *
 ***********************************************************************/

typedef struct 
{
    unsigned char reserved_1;
    unsigned char unit_ms;
    unsigned char hun_ten_ms;
    unsigned char tens_unit_sec;
    unsigned char tens_unit_min;
    unsigned char tens_unit_hours;
    unsigned char tens_unit_days;
    unsigned char hun_days;
    unsigned char tens_unit_years;
    unsigned char thou_hun_years;
    unsigned char reserved_2[2];
}preset_time_reg_t;

typedef struct
{
    unsigned char not_def[2];
    unsigned char status;
    unsigned char not_def1;
} hw_stat_reg_t;

typedef struct 
{
    unsigned char tens_unit_us;
    unsigned char unit_ms_hun_us;
    unsigned char hw_stat;
    unsigned char reserved_3;
    unsigned char hun_tens_ms;
    unsigned char tens_unit_sec;
    unsigned char tens_unit_min;
    unsigned char tens_unit_hours;
    unsigned char tens_unit_day;
    unsigned char status_hun_day;
    unsigned char tens_unit_year;
    unsigned char thou_hun_years;
}time_freeze_reg_t;

typedef unsigned char code_reg_t;

typedef struct
{

    unsigned char off_low;
    unsigned char off_high;
    unsigned char reserved_4[2];
}sync_gen_off_reg_t;

typedef struct
{

    unsigned char tens_unit_min;
    unsigned char tens_unit_hours;
    unsigned char sign_ascii;
    unsigned char reserved_5;
} local_off_t;

/*
 *  This structure can be used for both the position freeze
 *  and position preset registers.
 */
typedef struct
{

    unsigned char lat_tens_unit_degee;
    unsigned char lat_hun_degree;
    unsigned char lat_tens_unit_min;
    unsigned char lat_north_south;
    unsigned char lat_tenth_sec;
    unsigned char lat_tens_unit_sec;
    unsigned char long_tens_unit_degree;
    unsigned char long_hun_degree;
    unsigned char long_tens_unit_min;
    unsigned char long_east_west;
    unsigned char long_tenth_sec;
    unsigned char long_tens_unit_sec;
    unsigned char elv_tens_unit_km;
    unsigned char elv_sign;
    unsigned char elv_unit_tenth_m;
    unsigned char elv_hun_tens_m;
} pos_reg_t;

typedef struct
{

    unsigned char prn1_tens_units;
    unsigned char prn1_reserved;
    unsigned char lvl1_tenths_hundredths;
    unsigned char lvl1_tens_units;
    unsigned char prn2_tens_units;
    unsigned char prn2_reserved;
    unsigned char lvl2_tenths_hundredths;
    unsigned char lvl2_tens_units;
    unsigned char prn3_tens_units;
    unsigned char prn3_reserved;
    unsigned char lvl3_tenths_hundredths;
    unsigned char lvl3_tens_units;
    unsigned char prn4_tens_units;
    unsigned char prn4_reserved;
    unsigned char lvl4_tenths_hundredths;
    unsigned char lvl4_tens_units;
    unsigned char prn5_tens_units;
    unsigned char prn5_reserved;
    unsigned char lvl5_tenths_hundredths;
    unsigned char lvl5_tens_units;
    unsigned char prn6_tens_units;
    unsigned char prn6_reserved;
    unsigned char lvl6_tenths_hundredths;
    unsigned char lvl6_tens_units;
    unsigned char flag;
    unsigned char reserved[3];
} sig_levels_t;

typedef struct
{

    unsigned char tens_unit_us;
    unsigned char unit_ms_hun_us;
    unsigned char hun_tens_ms;
    unsigned char tens_unit_sec;
    unsigned char tens_unit_min;
    unsigned char tens_unit_hour;
    unsigned char tens_unit_days;
    unsigned char stat_hun_days;
    unsigned char tens_unit_year;
    unsigned char thou_hun_year;
    unsigned char reserved_5[2]; 
} ext_time_event_reg_t;

typedef struct
{

    unsigned char tens_unit_us;
    unsigned char unit_ms_hun_us;
    unsigned char hun_tens_ms;
    unsigned char tens_unit_sec;
    unsigned char tens_unit_min;
    unsigned char tens_unit_hour;
    unsigned char tens_unit_days;
    unsigned char mask_hun_days;
} time_cmp_reg_t;

typedef struct
{

    unsigned char err_stat;
    unsigned char no_def;
    unsigned char ossillator_stat[2];

} diag_reg_t;


typedef struct
{

    unsigned  res :2;
    unsigned  rate_int_mask :1;
    unsigned  cmp_int_mask  :1;
    unsigned  ext_int_mask  :1;
    unsigned  rate_stat_clr :1;
    unsigned  cmp_stat_clr  :1;
    unsigned  ext_stat_clr  :1;

    unsigned char reserved[3];

} hw_ctl_reg_t;

typedef struct
{

    unsigned  preset_pos_rdy    :1;
    unsigned  sel_pps_ref	    :1;
    unsigned  sel_gps_ref	    :1;
    unsigned  sel_time_code	    :1;
    unsigned  gen_stp_run	    :1;
    unsigned  preset_time_rdy   :1;
    unsigned  dst				:1;
    unsigned  mode_sel			:1;

    unsigned  ctl_am_dc			:1;
    unsigned  reserved			:3;
    unsigned  input_code	    :4;

    unsigned char rate_reserved;  
 
    unsigned  rate_flag			:4;
    unsigned  rate_reserved1    :4;
} conf_reg_t;

typedef struct
{

     unsigned char mem_reserved[0xf8];

     hw_ctl_reg_t	hw_ctl_reg;

     time_freeze_reg_t  time_freeze_reg;

     pos_reg_t		pos_freeze_reg;

     conf_reg_t		conf_reg;

     diag_reg_t		diag_reg;

     local_off_t	local_offset;

     sync_gen_off_reg_t sync_gen_offset;

     unsigned char 	reserved[4];

     unsigned char 	config_reg2_ctl;

     unsigned char      reserved2[11];

     time_cmp_reg_t	time_compare_reg;

     unsigned char 	reserved3[24];

     preset_time_reg_t	preset_time_reg;

     pos_reg_t		preset_pos_reg;

     ext_time_event_reg_t extern_time_event_reg;

     unsigned char 	reserved4[24];

     sig_levels_t	signal_levels_reg;

     unsigned char	reserved5[12];
} tt_mem_space_t;


#define MEM_DEV_PATH "/dev/ttime/0"

#define TTIME_MEMORY_SIZE 0x200

/*
 *  Defines for read commands
 */

#define TT_RD_FREEZE_REG	0x01
#define TT_RD_HW_CTL_REG	0x02
#define TT_RD_CNFG_REG		0x03
#define TT_RD_DIAG_REG		0x04
#define TT_RD_LCL_OFFSET	0x05
#define TT_RD_SYNC_GEN_OFF	0x06
#define TT_RD_CNFG_REG_2	0x07
#define TT_RD_TIME_CMP_REG	0x08
#define TT_RD_PRESET_REG	0x09
#define TT_RD_EXT_EVNT_REG	0x0a
#define TT_RD_SIG_LVL_REG	0x0b


/*
 *  Defines for write commands
 */
#define TT_WRT_FREEZE_REG	0x0c
#define TT_WRT_HW_CTL_REG	0x0d
#define TT_WRT_CNFG_REG		0x0e
#define TT_WRT_DIAG_REG		0x0f
#define TT_WRT_LCL_OFFSET	0x10
#define TT_WRT_SYNC_GEN_OFF	0x11
#define TT_WRT_CNFG_REG_2	0x12
#define TT_WRT_TIME_CMP_REG	0x13
#define TT_WRT_PRESET_REG	0x14
#define TT_WRT_EXT_EVNT_REG	0x15
#define TT_WRT_SIG_LVL_REG	0x16


/*
 *  Define the length of the buffers to move. 
 *  ( in 32 bit words )
 */

#define FREEZE_REG_LEN		7
#define HW_CTL_REG_LEN		1
#define CNFG_REG_1_LEN		1
#define DIAG_REG_LEN		1
#define LCL_OFFSET_LEN		1
#define SYNC_GEN_OFF_LEN	1
#define CNFG_REG_2_LEN		1
#define TIME_CMP_REG_LEN	2
#define PRESET_REG_LEN		7
#define EXT_EVNT_REG_LEN	3
#define	SIG_LVL_REG_LEN		7


