/*
 * ttime.h
 * 
 * This header file contains the necessary info for the driver.
 * API specific data will be in ttime_api.h
 */

#ifndef	_TTIME_H
#define	_TTIME_H

/* ioctl cmds - for user and driver, do not alter */

#define TTIME_HASH (('s'<<24) | ('c'<<16) | ('+'<<8))
#define TTIMEP_ENABLE_SIGURG  (TTIME_HASH+234)
#define TTIMEP_DISABLE_SIGURG (TTIME_HASH+235)
#define TTIMEP_WRITE_PBC      (TTIME_HASH+236)
#define TTIMEP_READ_PBC       (TTIME_HASH+237)

struct TTIMEio_t {           /* used with write/read PBC ioctls */
   int     offset;
   ulong_t value;
};

/* some PBC (pci bridge chip) offsets */

#define PCI_INT_STAT 0x48
#define PCI_INT_CFG  0x4c


/*
 * The following describes the structure of the  TTIMEnet+/pci card
 *
 *   16 MB - 0x00ffffff - top of TTIMEnet+ mem32 space 
 *   8  MB - 0x00800000 - TTIMEnet+ regs begins
 *   0  MB - 0x00000000 - TTIMEnet+ ram begins
 *
 */

/*
 * CSR offsets from register base (in words)
 * used by TTIMEnet.c and driver
 */
#define SCR_CSR0_OFFSET  0x001
#define SCR_CSR1_OFFSET  0x080
#define SCR_CSR2_OFFSET  0x103
#define SCR_CSR3_OFFSET  0x182
#define SCR_CSR4_OFFSET  0x205
#define SCR_CSR5_OFFSET  0x284
#define SCR_CSR6_OFFSET  0x307
#define SCR_CSR7_OFFSET  0x386
#define SCR_CSR8_OFFSET  0x409
#define SCR_CSR9_OFFSET  0x488
#define SCR_CSR10_OFFSET 0x50b
#define SCR_CSR11_OFFSET 0x58a
#define SCR_CSR12_OFFSET 0x60d
#define SCR_CSR13_OFFSET 0x68c
#define SCR_CSR14_OFFSET 0x70f
#define SCR_CSR15_OFFSET 0x78e

#define SCSR1_INT_ERR_BITS 0x0350
 


/* following defines are for the driver only */
#ifdef _KERNEL
#define TTIME_MAP_SIZE   0x200
#define TTIME_CTL_OFFSET 0x00800000
#define TTIME_IDENTITY "pci10b5,9050"
#endif




#endif	/* !_TTIME_H */
