/*
 * example -   respond to an interrupt generated by the 
 *             TrueTime 560 PCI card
 *
 *
 * Limitations:
 * - O2000 and Onyx2 only O2's will require a Kernel level driver
 *   to support interrupts.
 * - requires pciba device driver to be loaded:
 *   lboot -L pciba
 *
 * DISCLAIMER:
 * This software and its documentation are provided "AS IS" and without 
 * any expressed or implied warranties whatsoever.  No warranties as to 
 * performance, merchantability, or fitness for a particular purpose exist.
 * The user of this software is advised to test it thoroughly before relying 
 * on it.  The user must assume the entire risk and liability of using this
 * software.
 *
 */

#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <sys/prctl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/uli.h>

#define INTRPATH  "/hw/module/1/slot/io2/pci_xio/pci/2/intr"
#define MEMPATH  "/hw/module/1/slot/io2/pci_xio/pci/2/base/2_be"
#define MEMSIZE (512)

#define PCI_INTA 0x01
#define   BCDTOI(a)     (((a&0xf0) >> 4) * 10) + (a&0x0f)


extern int errno;

volatile int intr;
static void *ULIid;
volatile u_char *base_addr;
 
volatile int    *tfr_addr;
volatile u_char   *tfr_byte_addr;
union {
   volatile u_int   fint[3];
   volatile u_char  fbyte[12];
} freeze_regs;
int intr_old;


static void
intrfunc(void *arg)
{
     
     /*
	  *  Insert user interrupt processing code here.
	  *  NOTE: The user application is responsible for clearing
	  *  the interrupt by clearing the correct bits in the 
	  *  Hardware control register using either
	  *  write_hw_ctl_reg  or write_hw_ctl_reg_bitwise calls to the
	  *  API.
	  */
}

main(int argc, char *argv[])
{
    int fd;
    int fd_intr;
    int offset = 0x0;
    volatile unsigned char  *mem_addr;
    
    if (mlockall( MCL_CURRENT | MCL_FUTURE )){
        perror ("mlockall");
    }
    
    fd_intr = open(INTRPATH, O_RDWR);
    if (fd_intr < 0 ) {
        perror(INTRPATH);
        return errno;
    }

	if (init_560() ==0) /* API CALL */

        printf ("Opened successfully\n");
    
 
    /* Register the pci interrupt as a ULI source. */
    ULIid = (int *)ULI_register_pci(fd_intr,    /* the pci interrupt device */
                                    intrfunc,   /* the handler function     */
                                    0,          /* the argument to the handler*/
                                    0,          /* the # of semaphores needed */
                                    0,          /* the stack to use */
                                    0,          /* the stack size to use */
                                    PCI_INTA);  /* PCI line */
    if (ULIid == 0) {
        perror("register uli");
        exit(1);
    }

    printf ("Registered successfully for PCI INTA - sleep waiting for ints\n");

    
    while(intr < 100) {
    if (intr > intr_old) {
           intr_old = intr;
           /*
            * get time from time freeze registers (0xfc/4)
            */

		   
		   snapshot_time ((time_freeze_reg_t *)freeze_regs);
		   
           printf(" Time: %2.2d:%2.2d:%2.2d:%3.3d:%3.3d \n",
           BCDTOI(freeze_regs.fbyte[7]), BCDTOI(freeze_regs.fbyte[6]),
           BCDTOI(freeze_regs.fbyte[5]),
           ((freeze_regs.fbyte[1]&0xf0)>>4)+(BCDTOI(freeze_regs.fbyte[4]))*10,
           BCDTOI(freeze_regs.fbyte[0])+((freeze_regs.fbyte[1]&0x0f)*100));
           fflush (stdout);
        }
    }
}

