/******************************************************************
 *
 *  Module:  ttime_api.c
 *
 *  Author:  SIS  1998
 *
 *  Description
 *
 *       This module provides the actuall user interface to the
 *       true time 560-5900 timer board. All this interface does
 *       for this release is give the user access to the memory
 *       space of the board.
 *
 ******************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include "ttime_api.h"

/*
 *  Global structures.
 */

typedef union byteswap_u
{
    unsigned int long_word;
    unsigned char byte[4];
} byteswap_t;



    char		mem_path[256];
    char		int_path[256];
    int			fd;
    unsigned int 	*base_addr;

/*
 *  This is the structure that represents the memory of the
 *  true time card.
 */
    tt_mem_space_t	*tt_mem;


/*********************************************************************
 *
 *  init_560
 *
 *  This is the first routine that must be called by the in order
 *  to control the True Time board. This routine will open the 
 *  device and map the device memory into user space.
 *
 ********************************************************************/
int
init_560 ()
{

    caddr_t		membase;
    unsigned long	timeeval;

    FILE		*cfg_fd;

    char		string[256];

    int			ttime_int_flag;

    int			plx_fd;
    volatile int 	*plx_addr;

    /*
     *  Read the confguration file to pull out path names.
     */

     cfg_fd = fopen(TTIME_CONFIG, "rt");
     if (cfg_fd == NULL)
     {
         perror ("perror: ");
         return (-1);
     }

     while (fgets(string,128,cfg_fd) != NULL)
     {
         if ((string[0] == '#') || (strlen(string) == 0))
             continue;

         if (strncmp(string, "MEM_PATH=", 9) == 0)
             sscanf(string, "MEM_PATH=%s", mem_path);

         if (strncmp(string, "INT_PATH=", 9) == 0)
             sscanf(string, "INT_PATH=%s", int_path);

         if (strncmp(string, "ENABLE_INT", 10) == 0)
         {

             /*
              *  Enable interrupts on this device.
              */
             plx_fd = open (int_path, O_RDWR);
             if (plx_fd == -1)
             {
                 perror("open interrupt fail");
                 exit(-1);
             }
             else
             {
                 /*
                  * Grab plx configuration memory.
                  */
                 plx_addr = (volatile int *)(unsigned int *)mmap(0,128,
                            PROT_READ | PROT_WRITE, MAP_SHARED,
                            plx_fd,0);

                 if (plx_addr == (volatile int *)-1 )
                 {
                     perror ("mmap fail");
                     exit (-1);
                 }
                 *(volatile int *)(plx_addr + 0x13) |= 0x00000049;
             }

         }
     }

    /*
     *  Open the device
     */


     fd = open (mem_path, O_RDWR);

     if (fd == -1)
     {

         perror ("perror: ");
         return (-1);
     }


     /*
      * Map the device registers into user space.
      */

     membase = mmap ((caddr_t) 0, TTIME_MEMORY_SIZE,
		     	PROT_READ | PROT_WRITE,
		     	MAP_SHARED, fd, (off_t)0);

     if (membase == (caddr_t) -1)
     {
          perror ("perror: ");
          return (-1);
     }



    /*
     *  Map the memory.
     */

    tt_mem = (tt_mem_space_t *) membase;
    base_addr = (unsigned int *) membase;


    /*
     *  Return status to the user.
     */
     return (0);
}


void
close_560()
{

	close(fd);
}

/*************************************************************************
 *
 *
 *  read_diag_reg
 *  
 *  This routine is called to read the value of the diagnostic register.
 *  The diagnostic register stores the results of power up diagnostics 
 *  and the current setting of frequency control DAC
 *
 *
 **************************************************************************/
int
read_diag_reg (diag_reg_t *p_reg)
{

   unsigned int diag_reg;

   /*
    *  Read the value of the diagnostic register.
    */

    diag_reg = *(base_addr + DIAG_REG);

   /*
    *  Swap the bytes and return .
    */

   *(unsigned int *)p_reg = byte_swap(diag_reg);

   return (0);

}

/***************************************************************************
 *
 * write_hw_ctl_reg
 *
 * This routine allows the user to set the bits in the hardware
 * control register. The user and set each individual interrupt 
 * mask with this call. The interrupt masks are represented as single
 * bit bit fields. Refer to the 560-5900 users manual for details of this
 * register.
 *
 ****************************************************************************/

int
write_hw_ctl_reg (hw_ctl_reg_t *p_ctl)
{
 
	*(base_addr + HW_CTL_REG) = byte_swap(*(unsigned int *)p_ctl);

	return(0);
}
/********************************************************************
 *
 *  write_hw_ctl_reg_bitfield
 *
 *  This routine allows a user to access the fields of the hardware
 *  control register individually. Please refer to the 560-5900
 *  users manual for details of the hardware control register.
 *
 *********************************************************************/
int
write_hw_ctl_reg_bitfield (int bit_field, int value)
{
        union 
        {
   	    unsigned int ctl_reg_int;
            hw_ctl_reg_t ctl_reg_bit;
        } ctl_reg;

	/*
	 *  Read the current value of the hardware control register. 
	 */

        ctl_reg.ctl_reg_int = *(base_addr + HW_CTL_REG);

        /*
         *  Perform byte swap.
         */
        ctl_reg.ctl_reg_int = byte_swap(ctl_reg.ctl_reg_int);


        /*
         *  Set the correct bit.
         */

	switch (bit_field)
	{
	case HW_CTL_RATE_INT_MASK:
		ctl_reg.ctl_reg_bit.rate_int_mask = value;
		break;
	case HW_CTL_CMP_INT_MASK:
		ctl_reg.ctl_reg_bit.cmp_int_mask = value;
		break;
	case HW_CTL_EXT_INT_MASK:
		ctl_reg.ctl_reg_bit.ext_int_mask = value;
		break;
	case HW_CTL_RATE_GEN_INT_CLEAR:
		ctl_reg.ctl_reg_bit.rate_stat_clr = value;
		break;
	case HW_CTL_TIME_CMP_INT_CLEAR:
		ctl_reg.ctl_reg_bit.cmp_stat_clr = value;
		break;
	case HW_CTL_EXT_EVENT_INT_CLEAR:
		ctl_reg.ctl_reg_bit.ext_stat_clr = value;
		break;
	default:
		return (0xf);
	}

        /*
         *  Perform byte swap.
         */

        ctl_reg.ctl_reg_int = byte_swap(ctl_reg.ctl_reg_int);

        /*
         * Re-write the value of the control register.
         */
        *(base_addr + HW_CTL_REG) = ctl_reg.ctl_reg_int;


	return(0);
}


/**************************************************************************
 *
 * read_conf_reg
 *
 * This routine returns the value of the configuration register of the 
 * 560-5900. All 4 bytes of the configuration register are returned by
 * this call. The fields of the configuration register can be accessed 
 * as bit fields in the conf_reg_t structure. Refer to the 560-5900 users
 * manual for details of this bit field.
 *
 **************************************************************************/

int
read_conf_reg (conf_reg_t *p_conf)
{

    unsigned int configuration;

    configuration = *(base_addr + CONFIG_REG_1);

    *(unsigned int *)p_conf = byte_swap(configuration);

    return (0);
}


/************************************************************************
 *
 *
 *  read_conf_reg_bitfield
 *
 * This routine returns individual bit field values of the configuration 
 * register of the 560-5900. The fields of the configuration register can 
 * be accessed as bit fields in the conf_reg_t structure. Refer to the 
 * 560-5900 users manual for details of this bit field.
 *
 *
 *************************************************************************/
int
read_conf_reg_bitfield (int bit_field)
{

        union
        {
            conf_reg_t conf_reg;
            unsigned int config_reg_long;
        } configuration;


        configuration.config_reg_long = byte_swap(*(base_addr + CONFIG_REG_1));

	switch (bit_field)
	{
	case PRESET_POS_RDY_BIT:
		return(configuration.conf_reg.preset_pos_rdy);
	case SEL_1_PPS_REF_BIT:
		return(configuration.conf_reg.sel_pps_ref);
	case SEL_GPS_REF_BIT:
		return(configuration.conf_reg.sel_gps_ref);
	case SEL_TIME_CODE_REF_BIT:
		return(configuration.conf_reg.sel_time_code);
	case GEN_STOP_BIT:
		return(configuration.conf_reg.gen_stp_run);
	case PRESET_TIME_RDY_BIT:
		return(configuration.conf_reg.preset_time_rdy);
	case DST_BIT:
		return(configuration.conf_reg.dst);
	case MODE_SEL_BIT:
		return(configuration.conf_reg.mode_sel);
	case AM_DC_BIT:
		return(configuration.conf_reg.ctl_am_dc);
	case IN_CODE_SEL_BIT:
		return(configuration.conf_reg.input_code);
	case FLAG_BIT:
		return(configuration.conf_reg.rate_flag);
	default:
		break;
	}
        return (0xf);
}

/***********************************************************************************
 *
 * write_conf_reg
 *
 * This routine allows a user to write all 32 bits of the configuration register
 * at one time. The individual fields can be assessed as bit fields using the 
 * conf_reg_t structure.
 *
 ***********************************************************************************/
int
write_conf_reg (conf_reg_t *p_conf)
{
    int i; 

    *(base_addr + CONFIG_REG_1) = byte_swap(*(unsigned int *)p_conf);
/*
    for (i = 0; i<4; i++)
    {
        *tt_mem_t = *p_conf_t;
         tt_mem_t++;
         p_conf_t++;
    }
*/
    return(0);
}
/************************************************************************
 *
 *
 *  write_conf_reg_bitfield
 *
 *  This routine allows the user to set the fields of the configuration
 *  register individually.
 *
 *
 *************************************************************************/
int
write_conf_reg_bitfield (int bit_field, unsigned char value)
{
    union
    {
        conf_reg_t config_reg_bit;
        unsigned int config_reg_int;
    } configuration;
 
    configuration.config_reg_int = byte_swap (*(base_addr + CONFIG_REG_1));

	switch (bit_field)
	{
	case PRESET_POS_RDY_BIT:
		configuration.config_reg_bit.preset_pos_rdy = value;
		break;
	case SEL_1_PPS_REF_BIT:
		configuration.config_reg_bit.sel_pps_ref = value;
		break;
	case SEL_GPS_REF_BIT:
		configuration.config_reg_bit.sel_gps_ref = value;
		break;
	case SEL_TIME_CODE_REF_BIT:
		configuration.config_reg_bit.sel_time_code = value;
		break;
	case GEN_STOP_BIT:
		configuration.config_reg_bit.gen_stp_run = value;
		break;
	case PRESET_TIME_RDY_BIT:
		configuration.config_reg_bit.preset_time_rdy = value;
		break;
	case DST_BIT:
		configuration.config_reg_bit.dst = value;
		break;
	case MODE_SEL_BIT:
		configuration.config_reg_bit.mode_sel = value;
		break;
	case AM_DC_BIT:
		configuration.config_reg_bit.ctl_am_dc = value;
		break;
	case IN_CODE_SEL_BIT:
		configuration.config_reg_bit.input_code = value;
		break;
	case FLAG_BIT:
		configuration.config_reg_bit.rate_flag = value;
		break;
	}

        *(base_addr + CONFIG_REG_1) = byte_swap(configuration.config_reg_int);

	return(0);
}

/****************************************************************************
 *
 *  read_hw_stat_reg_bitfield
 *
 *  This routine allows the user to examine an individual bit of the
 *  hardware status register. This would allow an application to poll
 *  the TrueTime card for a specific event.
 *
 ****************************************************************************/

int
read_hw_stat_reg_bitfield(int bit_field)
{
        union 
        {
	    hw_stat_reg_t hw_stat;
            unsigned int stat;
        } time_status;

	/*
	 *  get a pointer to the hardware status register.
	 */
	time_status.stat = byte_swap(*(base_addr + HW_STAT_REG));

	switch (bit_field)
	{
	case HW_STAT_ANTENNA_SHORT:
		return(time_status.hw_stat.antenna_short_stat);
	case HW_STAT_ANTENNA_OPEN:
		return(time_status.hw_stat.antenna_open_stat);
	case HW_STAT_RATE_GEN_PULSE_STAT:
		return(time_status.hw_stat.rate_gen_pulse_stat);
	case HW_STAT_TIME_CMP_PULSE_STAT:
		return(time_status.hw_stat.time_cmp_pulse_stat);
	case HW_STAT_EXT_EVENT_STAT:
		return(time_status.hw_stat.ext_event_stat);
	default:
                break;

	}
	return(0xf);
}



/*******************************************************************************
 *
 *  read_local_offset_reg
 *
 *  This routine allows the user to read the 32 bit value of the local offset 
 *  register.
 *
 *  The local offset register is used to convertUTC-USNO to local time. THe range
 *  is from -12:59 to 12:59 hours. 
 *
 *******************************************************************************/

int
read_local_offset_reg (local_off_t *p_offset)
{
    unsigned int *p_offset_t;

    p_offset_t = (unsigned int *)p_offset;

    *p_offset_t = byte_swap(*(base_addr + LOCAL_OFF_REG));

    return (0);
}


/*********************************************************************************
 *
 *  write_local_offset_reg
 *
 *  This routine allows the user to write the 32 value of the local
 *  offset register.  The individual fields of the register can be 
 *  accessed using the local_off_t structure.
 *
 *  The local offset register is used to convertUTC-USNO to local time. The range
 *  is from -12:59 to 12:59 hours. 
 *
 **********************************************************************************/
int
write_local_offset_reg (local_off_t *p_offset)
{
    int i; 
    unsigned int *p_offset_t;

    p_offset_t = (unsigned int *)p_offset;

    *(base_addr + LOCAL_OFF_REG) = byte_swap(*p_offset_t);

    return(0);
}

/************************************************************************
 *
 *
 *  read_sync_offset_reg
 *
 *  When the sync generator is using amplitude modulation time code
 *  as the reference, there are inherent delays caused by signal 
 *  distribution amps or transmission media. The syncronized generator
 *  offset register is used to set the phase compensation value.
 *
 *  This routine allows the user to read the current value of the
 *  syncronized generator offset register.
 *
 *************************************************************************/

int
read_sync_offset_reg (sync_gen_off_reg_t *p_sync)
{
    unsigned int *p_sync_t;
    unsigned int *tt_mem_t;

    p_sync_t = (unsigned int *)p_sync;

    *p_sync_t = byte_swap (*(base_addr + SYNC_GEN_OFF_REG));

    return (0);
}

/************************************************************************
 *
 *
 *  write_sync_offset_reg
 *
 *  When the sync generator is using amplitude modulation time code
 *  as the reference, there are inherent delays caused by signal 
 *  distribution amps or transmission media. The syncronized generator
 *  offset register is used to set the phase compensation value.
 *
 *  This routine allows the user to write the current value of the
 *  syncronized generator offset register.
 *
 *************************************************************************/

int
write_sync_offset_reg (sync_gen_off_reg_t *p_sync)
{
    int i; 
    unsigned int *p_sync_t;

    p_sync_t = (unsigned int *)p_sync;
 
    *(base_addr + SYNC_GEN_OFF_REG) = byte_swap(*p_sync_t);

    return(0);
}


/***********************************************************************
 *
 *  read_time_cmp_reg
 *
 *  The time compare registers can be set to genterate a pulse at
 *  a predetermined time. This routine allows an application to read
 *  the value of the time compare registers, including the mask bit
 *  field which tells the 560-5900 which portion of the time compare 
 *  register to actually compare to the actuall time.
 *
 ***********************************************************************/

int
read_time_cmp_reg (time_cmp_reg_t *p_time)
{
    int i;
    unsigned int *p_time_t;

    p_time_t = (unsigned int *)p_time;

    for (i = 0; i<2; i++)
    {
        *p_time_t = byte_swap(*(base_addr + TIME_CMP_REG + i));
         p_time_t++;
    }

    return (0);
}

/***********************************************************************
 *
 *  write_time_cmp_reg
 *
 *  The time compare registers can be set to genterate a pulse at
 *  a predetermined time. This routine allows an application to write
 *  the value of the time compare registers, including the mask bit
 *  field which tells the 560-5900 which portion of the time compare 
 *  register to actually compare to the actuall time. REMEMBER to set the
 *  mask bits correctly in order to assure proper pulse times.
 *
 ***********************************************************************/

int
write_time_cmp_reg (time_cmp_reg_t *p_time)
{
    int i; 
    unsigned int *p_time_t;

    p_time_t = (unsigned int *)p_time;

    for (i = 0; i<2; i++)
    {
        *(base_addr + TIME_CMP_REG + i) = byte_swap(*p_time_t);
         p_time_t++;
    }

    return(0);
}


/*************************************************************************
 *
 *  read_preset_time_reg
 *
 *  The preset time registers are used to preset a starting time for
 *  the 560-5900. This routine allows the application to read the current
 *  value of the time preset register.
 *
 **************************************************************************/
int
read_preset_time_reg (preset_time_reg_t *p_time)
{
    int i;
    unsigned int *p_time_t;

    p_time_t = (unsigned int *)p_time;

    for (i = 0; i<3; i++)
    {
        *p_time_t = byte_swap (*(base_addr + PRESET_TIME_REG + i));
         p_time_t++;
    }

    return (0);
}

/*************************************************************************
 *
 *  write_preset_time_reg
 *
 *  The preset time registers are used to preset a starting time for
 *  the 560-5900. This routine allows the application to set the current
 *  value of the time preset register.
 *
 **************************************************************************/
int
write_preset_time_reg (preset_time_reg_t *p_time)
{
    int i; 

    unsigned int *p_preset_time;

    p_preset_time = (unsigned int *)p_time;

    for (i = 0; i< 3; i++)
    {
         *(base_addr + PRESET_TIME_REG + i) = byte_swap(*p_preset_time);
         p_preset_time ++;
    }
    write_conf_reg_bitfield (PRESET_TIME_RDY_BIT,1);

    return(0);
}

/**************************************************************************
 *
 *  reset_time
 *
 *  inputs: NONE
 *
 *  This routine zeros out the preset time registers.
 *
 ***************************************************************************/
int
reset_time ()
{

    int i;

    for (i = 0; i< 3; i++)
    {
         *(base_addr + PRESET_TIME_REG + i) = 0;
    }
    write_conf_reg_bitfield (PRESET_TIME_RDY_BIT,1);
    return (0);
}

/**************************************************************************
 *
 *  set_new_time
 *
 *  inputs: NONE
 *
 *  This routine sets the preset time registers.
 *
 ***************************************************************************/
int
set_new_time (preset_time_reg_t *p_time)
{

    int i;
    unsigned int *p_preset_time;

    p_preset_time = (unsigned int *)p_time;

    for (i = 0; i< 3; i++)
    {
         *(base_addr + PRESET_TIME_REG + i) = byte_swap(*p_preset_time);
         p_preset_time ++;
    }
    write_conf_reg_bitfield (PRESET_TIME_RDY_BIT,1);
    return (0);
}

/*************************************************************************
 *
 *  read_preset_position_reg
 *
 *  The preset position registers are used to preset a starting GPS position for
 *  the 560-5900. This routine allows the application to read the current
 *  value of the position preset register.
 *
 **************************************************************************/

int
read_preset_position_reg (pos_reg_t *p_position)
{
    int i; 
    unsigned int *p_position_t;

    p_position_t = (unsigned int *)p_position;

    for (i = 0; i<4; i++)
    {
        *p_position_t = byte_swap (*(base_addr + PRESET_POS_REG + i));
         p_position_t++;
    }

    return (0);
}

/*************************************************************************
 *
 *  write_preset_position_reg
 *
 *  The preset position registers are used to preset a starting GPS position for
 *  the 560-5900. This routine allows the application to write the current
 *  value of the position preset register.
 *
 **************************************************************************/

int
write_preset_position_reg (pos_reg_t *p_position)
{
    int i; 
    unsigned int *p_position_t;

    p_position_t = (unsigned int *)p_position;

    for (i = 0; i<4; i++)
    {
        *(base_addr + PRESET_POS_REG + i) = *p_position_t;
         p_position_t++;
    }

    return(0);
}

/************************************************************************
 *
 * read_external_event_reg
 *
 * The external event registers contain the time captured when a pulse
 * is generated on PIN 1 of the 9-pin connector. A flag in the hardware
 * status register is set when the external time is available. (this may 
 * take 2ms)  the time in the register is accurate within 3us.
 *
 ************************************************************************/
int
read_external_event_reg (ext_time_event_reg_t *p_time)
{
    int i;
    unsigned int *p_time_t;

    p_time_t = (unsigned int *)p_time;

    for (i = 0; i<3; i++)
    {
        *p_time_t = byte_swap(*(base_addr + EXT_EVENT_REG + i));
         p_time_t++;
    }

    return (0);
}

/******************************************************************************
 *
 *  read_signal_level_reg
 *
 *  Signal levels are provided for up to 6 satellites. Both PRN and signal
 *  levels are provided. This routine allows the user to access the signal 
 *  level registers. The Flag byte indicates that information is not available
 *  or is being updated.
 *
 ******************************************************************************/
int
read_signal_level_reg (sig_levels_t *p_levels)
{
    int i;
    unsigned int *p_levels_t;
    unsigned char *tt_mem_t;

    p_levels_t = (unsigned int *)p_levels;

    for (i = 0; i<7; i++)
    {
        *p_levels_t = byte_swap(*(base_addr + SIG_LVL_PRN1 + i));
         p_levels_t++;
    }

    return (0);
}


/*********************************************************************
 *
 *  freeze_time
 *
 *  this routine will allow the user to freeze the time in the time
 *  freeze registers. The current time will remain in the time freeze
 *  registers until the user calls this routine again or calls 
 *  snapshot_time which will freeze the time and read it all in one call.
 *
 *********************************************************************/
int
freeze_time()
{

	*(unsigned char *)(tt_mem + TIME_FREEZE_REG) = 0;

	return (0);
}

int
snapshot_time (time_freeze_reg_t *p_time)
{
	int i;
	unsigned int *p_time_t, *tt_mem_t;

        p_time_t = (unsigned int *)p_time;
        tt_mem_t = (base_addr + TIME_FREEZE_REG);

	/*
	 *  First write to the freeze reg to freeze the time.
	 */
	*tt_mem_t = 0xffffffff;
	*tt_mem_t = 0xffffffff;
	*tt_mem_t = 0xffffffff;


	/* 
	 *  Now read the time sequentially
	 */
	for (i=0; i < 3; i++)
	{
		*p_time_t = byte_swap(*tt_mem_t);
		p_time_t++;
		tt_mem_t++;
	}

	return (0);

}


int
read_position_freeze_reg (pos_reg_t *p_time)
{
	int i;
	unsigned long *p_time_t, *tt_mem_t;

	/*
	 *  assigne temporary pointers
	 */
	p_time_t = (unsigned long *)p_time;
	tt_mem_t = (unsigned long *)&tt_mem->pos_freeze_reg;


	/*
	 *  First write to the freeze reg to freeze the position.
	 */
	*(base_addr + TIME_FREEZE_REG) = 0xff;
	*(base_addr + TIME_FREEZE_REG) = 0xff;
	*(base_addr + TIME_FREEZE_REG) = 0xff;

	/* 
	 *  Now read the time sequentially
	 */
	for (i=0; i < 4; i++)
	{
		*p_time_t = byte_swap(*tt_mem_t);
		p_time_t++;
		tt_mem_t++;
	}

	return (0);

}
/******************************************************************
 *
 *  enable_device_interrupts
 *
 *  Inputs: None
 *
 *  Outputs:	0:  Success
 *				-1: Fail
 *
 *  This routine allows an application to enable interrupts from the 
 *  TrueTime card. This is required above and beyond setting the bits
 *  int the hardware control register. This routine enables interrupts on
 *  the PLX chip.
 *
 *
 **************************************************************************/
unsigned int
enable_device_interrupts()
{
    return (0);
}

/*****************************************************************************8
 *
 *  byte_swap
 *
 *  Inputs: 32 bit integer
 *
 *  Output: byte swapped 32 bit integer.
 *
 *  This routine will be used to compensate for the byte allignment
 *  differences between big-endian and little-endian integers.
 *
 *****************************************************************************/
unsigned int
byte_swap(unsigned int input_num)
{

    byteswap_t byte_swap;
    unsigned char temp;

    byte_swap.long_word = input_num;

    temp = byte_swap.byte[3];
    
    byte_swap.byte[3] = byte_swap.byte[0];

    byte_swap.byte[0] = temp;

    temp = byte_swap.byte[2];

    byte_swap.byte[2] = byte_swap.byte[1];
  
    byte_swap.byte[1] = temp;

    return (byte_swap.long_word);

}
